@@ -20,7 +20,7 @@ static void popup_error_message (void)
 				   "indicate a problem with Bonobo, or a non-GNOME (e.g. KDE) settings manager may already "
 				   "be active and conflicting with the GNOME settings manager."));
 
-  gtk_widget_show (dialog);
+  gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 }
 
@@ -40,19 +40,15 @@ activate_settings_daemon (void)
   object = bonobo_activation_activate_from_id  ("OAFIID:GNOME_SettingsDaemon",
 						0, NULL, &ev);
   
-  if (ev._major != CORBA_NO_EXCEPTION) {
-    popup_error_message ();
-    return FALSE;
-  }
-  
-  if (object == CORBA_OBJECT_NIL) {
+  if (BONOBO_EX(&ev) || object == CORBA_OBJECT_NIL ) {
     popup_error_message ();
+    CORBA_exception_free(&ev);
     return FALSE;
   }
 
   /*bool = GNOME_SettingsDaemon_awake (corba_foo, "MyService", &ev);
     printf ("bool is %d\n", bool);*/
 
-
+  CORBA_exception_free(&ev);
   return TRUE;
 }
