@@ -27,9 +27,15 @@
 #include "config.h"
 #include "acme-volume-gstreamer.h"
 
+#ifdef HAVE_GST10
+#include <gst/gst.h>
+#include <gst/interfaces/mixer.h>
+#include <gst/interfaces/propertyprobe.h>
+#else
 #include <gst/gst.h>
 #include <gst/mixer/mixer.h>
 #include <gst/propertyprobe/propertyprobe.h>
+#endif
 
 #include <string.h>
 
@@ -40,6 +46,7 @@ struct AcmeVolumeGStreamerPrivate
   	GstMixer      *mixer;
   	GstMixerTrack *track;
  	guint timer_id;
+	gint state;
 };
 
 static GObjectClass *parent_class = NULL;
@@ -87,52 +94,79 @@ acme_volume_gstreamer_set_mute (AcmeVolume *vol, gboolean val)
 	gst_mixer_set_mute (self->_priv->mixer,
 			    self->_priv->track,
 			    val);
-	
+
+	if (val)
+		self->_priv->state |= 1;
+	else {
+		GstMixerTrack *track = self->_priv->track;
+		gint *volumes, n;
+
+		self->_priv->state &= ~1;
+		volumes = g_new0 (gint, track->num_channels);
+		for (n = 0; n < track->num_channels; n++)
+			volumes[n] = (self->_priv->state >> 1) /
+				track->num_channels;
+		gst_mixer_set_volume (self->_priv->mixer, track, volumes);
+		g_free (volumes);
+	}
+
 	acme_volume_gstreamer_close (self);
 }
 
+static void
+update_state (AcmeVolumeGStreamer * self)
+{
+	gint *volumes, vol = 0, n;
+	GstMixerTrack *track = self->_priv->track;
+
+	/* update mixer by getting volume */
+	volumes = g_new0 (gint, track->num_channels);
+	gst_mixer_get_volume (self->_priv->mixer, track, volumes);
+	for (n = 0; n < track->num_channels; n++)
+		vol += volumes[n];
+	g_free (volumes);
+
+	/* update mute flag, and volume if not muted */
+	if (GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_MUTE) ||
+	    (vol == 0 && (self->_priv->state >> 1) != 0))
+		self->_priv->state |= 1;
+	else
+		self->_priv->state = vol << 1;
+}
+
 static gboolean
 acme_volume_gstreamer_get_mute (AcmeVolume *vol)
 {
 	AcmeVolumeGStreamer *self = (AcmeVolumeGStreamer *) vol;
-	gboolean mute;
 
 	if (acme_volume_gstreamer_open (self) == FALSE)
 		return FALSE;
 
-	mute = GST_MIXER_TRACK_HAS_FLAG (self->_priv->track,
-					 GST_MIXER_TRACK_MUTE);
-
+	update_state (self);
 	acme_volume_gstreamer_close (self);
 
-	return mute;
+	return (self->_priv->state & 1);
 }
 
 static int
 acme_volume_gstreamer_get_volume (AcmeVolume *vol)
 {
-	gint i, vol_total = 0, *volumes;
 	double volume;
 	AcmeVolumeGStreamer *self = (AcmeVolumeGStreamer *) vol;
 	GstMixerTrack *track;
 	
 	if (acme_volume_gstreamer_open (self) == FALSE)
 		return 0;
-
 	track = self->_priv->track;
 
-	volumes = g_new0 (gint, track->num_channels);
-	gst_mixer_get_volume (self->_priv->mixer, track, volumes);
-	for (i = 0; i < track->num_channels; ++i)
-		vol_total += volumes[i];
-	g_free (volumes);
+	update_state (self);
+
+	/* normalize to [0,100] scale that acme wants */
+	volume = (self->_priv->state >> 1) / (double) track->num_channels;
+	volume = 100 * (volume - track->min_volume) /
+		(track->max_volume - track->min_volume);
 
 	acme_volume_gstreamer_close (self);
-	
-	volume = vol_total / (double)track->num_channels;
-
-	/* Normalize the volume to the [0, 100] scale that acme expects. */
-	volume = 100 * (volume - track->min_volume) / (track->max_volume - track->min_volume);
 
 	return (gint) volume;
 }
@@ -140,8 +174,7 @@ acme_volume_gstreamer_get_volume (AcmeVolume *vol)
 static void
 acme_volume_gstreamer_set_volume (AcmeVolume *vol, int val)
 {
-	gint i, *volumes;
-	double volume;
+	gint i, *volumes, volume;
 	AcmeVolumeGStreamer *self = (AcmeVolumeGStreamer *) vol;
 	GstMixerTrack *track;
 
@@ -152,7 +185,8 @@ acme_volume_gstreamer_set_volume (AcmeVolume *vol, int val)
 	val = CLAMP (val, 0, 100);
 
 	/* Rescale the volume from [0, 100] to [track min, track max]. */
-	volume = (val / 100.0) * (track->max_volume - track->min_volume) + track->min_volume;
+	volume = (val / 100.0) * (track->max_volume - track->min_volume) +
+		track->min_volume;
 
 	volumes = g_new (gint, track->num_channels);
 	for (i = 0; i < track->num_channels; ++i)
@@ -160,6 +194,10 @@ acme_volume_gstreamer_set_volume (AcmeVolume *vol, int val)
 	gst_mixer_set_volume (self->_priv->mixer, track, volumes);
 	g_free (volumes);
  	
+	/* update state */
+	self->_priv->state = (self->_priv->state & 1) |
+		((volume * track->num_channels) << 1);
+
  	acme_volume_gstreamer_close (self);
 }
  
@@ -182,14 +220,55 @@ acme_volume_gstreamer_close_real (AcmeVolumeGStreamer *self)
 	return FALSE;
 }
 
+#ifdef HAVE_GST10
+/*
+ * _acme_set_mixer
+ * Arguments: mixer - pointer to mixer element
+ *            data - pointer to user data (AcmeVolumeGStreamer to be modified)
+ * Returns: gboolean indicating success
+ */
+static gboolean
+_acme_set_mixer(GstMixer *mixer, gpointer user_data)
+{
+   const GList *tracks;
+
+   tracks = gst_mixer_list_tracks (mixer);
+
+   while (tracks != NULL) {
+      GstMixerTrack *track = GST_MIXER_TRACK (tracks->data);
+
+      if (GST_MIXER_TRACK_HAS_FLAG (track, GST_MIXER_TRACK_MASTER)) {
+         AcmeVolumeGStreamer *self;
+
+         self = ACME_VOLUME_GSTREAMER (user_data);
+
+         self->_priv->mixer = mixer;
+         self->_priv->track = track;
+
+         /* no need to ref the mixer element */
+         g_object_ref (self->_priv->track);
+         return TRUE;
+      }
+
+      tracks = tracks->next;
+   }
+
+   return FALSE;
+}
+#endif
+
 /* This is a modified version of code from gnome-media's gst-mixer */
 static gboolean
 acme_volume_gstreamer_open (AcmeVolumeGStreamer *vol)
 {
   	AcmeVolumeGStreamer *self = (AcmeVolumeGStreamer *) vol;
+#ifdef HAVE_GST10
+	GList *mixer_list;
+#else
   	const GList *elements;
   	gint num = 0;
-  
+#endif
+
 	if (self->_priv == NULL)
 		return FALSE;
 	
@@ -205,9 +284,22 @@ acme_volume_gstreamer_open (AcmeVolumeGStreamer *vol)
 	 * for first one named "volume".
 	 * 
 	 * We should probably do something intelligent if we don't find an
-	 * appropriate mixer/track.  But now we do something stupid... everything
-	 * just becomes a no-op.
+	 * appropriate mixer/track.  But now we do something stupid...
+	 * everything just becomes a no-op.
 	 */
+#ifdef HAVE_GST10
+   mixer_list = gst_audio_default_registry_mixer_filter (_acme_set_mixer,
+                                                          TRUE,
+                                                          self);
+
+   if (mixer_list == NULL)
+      return FALSE;
+
+   /* do not unref the mixer as we keep the ref for self->priv->mixer */
+   g_list_free (mixer_list);
+
+   return TRUE;
+#else
 	elements = gst_registry_pool_feature_list (GST_TYPE_ELEMENT_FACTORY);
 	for ( ; elements != NULL && self->_priv->mixer == NULL; elements = elements->next) {
 		GstElementFactory *factory = GST_ELEMENT_FACTORY (elements->data);
@@ -289,6 +381,7 @@ acme_volume_gstreamer_open (AcmeVolumeGStreamer *vol)
 		g_free (title);
 	}
 	return FALSE;
+#endif
 }
 
 static void
